#!/usr/bin/env python3

from os import path, remove as rmfile
from shutil import copytree, copy2, rmtree as rmdir

from glob import glob

from .logger import log


def exists(f: str):
    return path.exists(f)


def isdir(f: str):
    return path.isdir(f)


def isfile(f: str):
    return path.isfile(f)


def get_files(pattern):
    return glob(pattern)


def remove(f):
    try:
        if exists(f):
            if isdir(f):
                return rmdir(f, ignore_errors=True)
            else:
                return rmfile(f)
    except Exception as err:
        log('remove', f, err)
        pass
    return False


def copy(source, dest):
    try:
        if exists(source):
            if isdir(source):
                return copytree(source, dest)
            else:
                return copy2(source, dest)
    except Exception as err:
        log('copy', 'from', source, 'to', dest, err)
        pass
    return True



#!/usr/bin/env python3

from .modules import fs

from .modules.detector import is_int, is_num, is_float, is_str, \
    is_bool, is_list, is_dict, is_valid_url

from .modules.text import genid, slugify, remove_tags, pluralize, \
    md5, md160, sha256

from .modules.utime import get_time, get_local_time, get_utc_time, \
    format_time, PY_DATE_PATTERN, MY_DATE_PATTERN

from .modules.utils import throttle, timing, jprint, \
    write_json_to_file, read_json_from_file, \
    byte_to_text, get_base_url, has_installed, \
    compose, pipe, curry


def version():
    f = os.path.dirname(os.path.realpath(__file__))
    return open(f + '/VERSION').read().strip()



#!/usr/bin/env python3

from typing import Any
from urllib.parse import urlparse


def is_int(val: Any):
    try:
        return int(val) == val and '.' not in str(val)
    except Exception:
        pass
    return False


def is_num(val: Any):
    try:
        return float(val) == val
    except Exception:
        pass
    return False


def is_float(val: Any):
    return is_num(val) and not is_int(val)


def is_str(val: Any):
    try:
        return str(val) == val
    except Exception:
        pass
    return False


def is_bool(val: Any):
    return val is True or val is False


def is_list(val: Any):
    try:
        return isinstance(val, list)
    except Exception:
        pass
    return False


def is_dict(val: Any):
    try:
        return isinstance(val, dict)
    except Exception:
        pass
    return False


def is_valid_url(url: str):
    try:
        result = urlparse(url)
        return all([result.scheme, result.netloc])
    except Exception:
        pass
    return False



#!/usr/bin/env python3

import re
import string
import hashlib
import binascii

from random import shuffle, choice

from .logger import log

TAG_RE = re.compile(r'<[^>]+>')


def genid(count: int = 16, prefix: str = ''):
    alphadigits = string.ascii_letters + string.digits
    alphadigits_list = list(alphadigits)
    shuffle(alphadigits_list)
    alphadigits = ''.join(alphadigits_list)
    leng = count - len(prefix)
    if leng < 0:
        return prefix
    return prefix + ''.join([choice(alphadigits) for n in range(leng)])


def get_char_map():
    char_map = {
        'a': 'á|à|ả|ã|ạ|ă|ắ|ặ|ằ|ẳ|ẵ|â|ấ|ầ|ẩ|ẫ|ậ|ä',
        'A': 'Á|À|Ả|Ã|Ạ|Ă|Ắ|Ặ|Ằ|Ẳ|Ẵ|Â|Ấ|Ầ|Ẩ|Ẫ|Ậ|Ä',
        'c': 'ç',
        'C': 'Ç',
        'd': 'đ',
        'D': 'Đ',
        'e': 'é|è|ẻ|ẽ|ẹ|ê|ế|ề|ể|ễ|ệ|ë',
        'E': 'É|È|Ẻ|Ẽ|Ẹ|Ê|Ế|Ề|Ể|Ễ|Ệ|Ë',
        'i': 'í|ì|ỉ|ĩ|ị|ï|î',
        'I': 'Í|Ì|Ỉ|Ĩ|Ị|Ï|Î',
        'o': 'ó|ò|ỏ|õ|ọ|ô|ố|ồ|ổ|ỗ|ộ|ơ|ớ|ờ|ở|ỡ|ợ|ö',
        'O': 'Ó|Ò|Ỏ|Õ|Ọ|Ô|Ố|Ồ|Ổ|Ô|Ộ|Ơ|Ớ|Ờ|Ở|Ỡ|Ợ|Ö',
        'u': 'ú|ù|ủ|ũ|ụ|ư|ứ|ừ|ử|ữ|ự|û',
        'U': 'Ú|Ù|Ủ|Ũ|Ụ|Ư|Ứ|Ừ|Ử|Ữ|Ự|Û',
        'y': 'ý|ỳ|ỷ|ỹ|ỵ',
        'Y': 'Ý|Ỳ|Ỷ|Ỹ|Ỵ'
    }
    xmap = {}
    for k in char_map:
        arr = char_map[k].split('|')
        for key in arr:
            xmap[key] = k
    return xmap


CHARS_MAP = get_char_map()


def strip_accents(text: str):
    def check_and_replace(c):
        if c in CHARS_MAP:
            return CHARS_MAP[c]
        return c

    mod = [check_and_replace(a) for a in text]
    return ''.join(mod)


def slugify(text: str):
    text = strip_accents(text).lower()
    return re.sub(r'[\W_]+', '-', text)


def remove_tags(html: str):
    return TAG_RE.sub('', html)


def _xhash(txt='', algorithm='md5'):
    if algorithm == 'ripemd160':
        m = hashlib.new('ripemd160')
    else:
        m = hashlib.md5()
    m.update(str(txt).encode('utf-8'))
    return m.hexdigest()


def md5(txt: str):
    if not txt:
        return ''
    return _xhash(str(txt), md5)


def md160(txt: str):
    if not txt:
        return ''
    return _xhash(str(txt), 'ripemd160')


def sha256(txt: str, salt: str, iterations: int = 50000, dkpower: int = 3):
    dk = hashlib.pbkdf2_hmac(
        'sha256',
        str(txt).encode('utf8'),
        'salt'.encode('utf8'),
        iterations,
        2 ** dkpower
    )
    return binascii.hexlify(dk).decode('ascii')


# pluralize(word, count):
# return: plural form of a countable noun depending on its quantity
# function customized from the following script:
# http://code.activestate.com/recipes/577781-pluralize-word-convert-singular-word-to-its-plural/

ABERRANT_PLURAL_MAP = {
    'antenna': 'algae',
    'appendix': 'appendices',
    'baggage': 'baggage',
    'barracks': 'barracks',
    'corpus': 'corpora',
    'child': 'children',
    'deer': 'deer',
    'focus': 'foci',
    'fungus': 'fungi',
    'furniture': 'furniture',
    'genus': 'genera',
    'goose': 'geese',
    'index': 'indices',
    'information': 'information',
    'larva': 'larvae',
    'luggage': 'luggage',
    'man': 'men',
    'mouse': 'mice',
    'news': 'news',
    'nucleus': 'nuclei',
    'person': 'people',
    'syllabus': 'syllabi',
    'vertebra': 'vertebrae',
    'woman': 'women',
}

VOWELS = set('aeiou')

NORMAL_O_ENDS = [
    'auto',
    'kilo',
    'memo',
    'photo',
    'piano',
    'pimento',
    'pro',
    'solo',
    'soprano'
]


def pluralize(word: str = None, count: int = 1):
    if not word:
        return ''
    if count <= 1 or len(word) < 3:
        return word

    plural = ABERRANT_PLURAL_MAP.get(word, None)
    if plural:
        return plural

    try:
        if word[-1] == 'y' and word[-2] not in VOWELS:
            return word[:-1] + 'ies'
        if word[-2:] == 'is':
            return word[:-2] + 'es'
        if word[-2:] == 'us':
            return word[:-2] + 'i'
        if word[-2:] == 'on' or word[-2:] == 'um':
            return word[:-2] + 'a'
        if word[-1] == 'x':
            return word + 'es'
        if word[-1] == 'o':
            if word in NORMAL_O_ENDS or word[-2] in VOWELS:
                return word + 's'
            return word + 'es'
        if word[-1] == 'f':
            return word[:-1] + 'ves'
        if word[-2:] == 'fe':
            return word[:-2] + 'ves'
        if word[-1] == 's':
            if word[-2] in VOWELS:
                if word[-3:] == 'ius':
                    return word[:-2] + 'ises'
                return word[:-1] + 'ses'
            return word + 'es'
        if word[-2:] in ('ch', 'sh'):
            return word + 'es'
    except Exception as err:
        log('pluralize', err)
        pass
    return word + 's'



#!/usr/bin/env python3

from time import time
from datetime import datetime

from .detector import is_num

PY_DATE_PATTERN = '%Y-%m-%d %H:%M:%S %z'
MY_DATE_PATTERN = '%a, %b %d, %Y %H:%M:%S'


def get_time(ms=False):
    t = time()
    return t if ms is False else int(t)


def format_time(d=datetime.now(), pattern=PY_DATE_PATTERN):
    if is_num(d):
        d = datetime.fromtimestamp(d)
    return d.strftime(pattern)


def get_local_time():
    return format_time(datetime.now())


def get_utc_time():
    return format_time(datetime.utcnow())



#!/usr/bin/env python3

from setuptools import setup


PACKAGE = 'bella'
VERSION = open(PACKAGE + '/VERSION').read().strip()
README = open('README.md').read().strip()


setup(
    name=PACKAGE,
    version=VERSION,
    packages=[PACKAGE],

    description="BellaPy - A useful helper for any python program",
    long_description=README,
    long_description_content_type="text/markdown",

    url='https://github.com/ndaidong/bellapy',

    author='@ndaidong',
    author_email='ndaidong@gmail.com',

    license='MIT',

    classifiers=[
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities'
    ],

    keywords='detection, manipulation, datetime, utils, helpers',

    install_requires=[],

    extras_require={
        'test': ['pycodestyle', 'safety', 'pytest'],
    },
    include_package_data=True,

    zip_safe=False,

    entry_points={
        'console_scripts': [
            'bella=bella:init'
        ]
    }
)



#!/usr/bin/env python3

import time
import json
from typing import Any
from functools import reduce
from urllib.parse import urlparse
from importlib.util import find_spec
from json import dumps, load

from .logger import log
from .fs import exists


def throttle(seconds: int):
    def decorate(f):
        t = None

        def wrapped(*args, **kwargs):
            nonlocal t
            t_ = time.time()
            if t is None or t_ - t >= s:
                result = f(*args, **kwargs)
                t = time.time()
                return result
        return wrapped
    return decorate


def timing(name: str):
    def caller(f):
        def wrap(*args):
            start = time.time()
            ret = f(*args)
            end = time.time()
            ms = end - start
            print('Timing for `{}`: {}'.format(
                name,
                str(round(ms, 2)) + ' s'
            ))
            return ret
        return wrap
    return caller


def write_json_to_file(file_path: str = '', data: dict = {}):
    try:
        jstr = dumps(data, indent=2, sort_keys=True, ensure_ascii=False)
        with open(file_path, 'w') as write_file:
            write_file.write(jstr)
            write_file.flush()
        return True
    except Exception as err:
        log('write_json_to_file', file_path, err)
        pass
    return False


def read_json_from_file(file_path: str = ''):
    try:
        if exists(file_path):
            with open(file_path, 'r') as read_file:
                return load(read_file)
    except Exception as err:
        log('read_json_from_file', file_path, err)
        pass
    return {}


def jprint(data: Any, sorting=True, identation=2):
    try:
        if type(data) is str:
            data = json.loads(data)
        return print(json.dumps(
            data, sort_keys=sorting, indent=identation
        ))
    except Exception as err:
        log('jprint', err)
        pass
    return print(data)


def byte_to_text(bytesize, precision=2):
    abbrevs = (
        (1 << 50, 'PB'),
        (1 << 40, 'TB'),
        (1 << 30, 'G'),
        (1 << 20, 'M'),
        (1 << 10, 'K'),
        (1, 'bytes')
    )
    if bytesize == 1:
        return '1 byte'
    for factor, suffix in abbrevs:
        if bytesize >= factor:
            break
    if factor == 1:
        precision = 0
    result = bytesize / float(factor)
    if result <= 0:
        return 0
    return '%.*f %s' % (precision, result, suffix)


def get_base_url(url):
    try:
        result = urlparse(url)
        return '://'.join([result.scheme, result.netloc])
    except Exception as err:
        log('get_base_url', url)
        pass
    return url


def has_installed(pkg):
    return find_spec(pkg) is not None


def compose(*fns):
    return reduce(
        lambda f, g: lambda x: f(g(x)),
        fns,
        lambda x: x
    )


def pipe(*fns):
    return reduce(
        lambda f, g: lambda x: g(f(x)),
        fns,
        lambda x: x
    )


def curry(func):
    def curried(*args, **kwargs):
        if len(args) + len(kwargs) >= func.__code__.co_argcount:
            return func(*args, **kwargs)
        return (lambda *args2, **kwargs2:
                curried(*(args + args2), **dict(kwargs, **kwargs2)))
    return curried
